#import "NSTableView+util.h"
#import "SwappingTableView.h"
#import "SortingDelegate.h"

#import "SortingDelegate.h"

// ----------------------------------------------------------------------------------------
// Sorting Support
// ----------------------------------------------------------------------------------------

typedef struct { NSString *key; BOOL descending; } SortContext;

// Sort array of itemNums, by looking up the itemNum in the dictionary of dictionaries.
// based on code of Ondra Cada <ocs@ocs.cz> on cocoa-dev list

int ORDER_BY_CONTEXT(id left, id right, void *ctxt)
{
	SortContext *context = (SortContext*)ctxt;
	int order = 0;
	id key = context->key;
	if (0 != key)
	{
		id first,second;	// the actual objects to compare

		if (context->descending)
		{
			first  = [right objectForKey:key];
			second = [left  objectForKey:key];
		}
		else
		{
			first  = [left  objectForKey:key];
			second = [right objectForKey:key];
		}

		if ([first respondsToSelector:@selector(caseInsensitiveCompare:)])
		{
			order = [first caseInsensitiveCompare:second];
		}
		else	// sort numbers or dates
		{
			order = [(NSNumber *)first compare:second];
		}
	}
	return order;
}

@implementation SortingDelegate

// ----------------------------------------------------------------------------------------
// Memory
// ----------------------------------------------------------------------------------------

- (void)dealloc
{
	[mSortingColumn release];
	[mSortingKey release];
	[super dealloc];
}

// ----------------------------------------------------------------------------------------
// Accessors
// ----------------------------------------------------------------------------------------

- (void) setSortingColumn:(NSTableColumn *)inNewValue
{
	[inNewValue retain];
	[mSortingColumn release];
	mSortingColumn = inNewValue;
}

- (void) setSortingKey:(NSString *)inNewValue
{
	[inNewValue retain];
	[mSortingKey release];
	mSortingKey = inNewValue;
}

// ----------------------------------------------------------------------------------------
// Selection save/restore
// ----------------------------------------------------------------------------------------

- (NSSet *) saveSelectionFromTable:(NSTableView *)inTableView
{
	NSMutableSet *result = [NSMutableSet set];
	NSEnumerator *theEnum = [inTableView selectedRowEnumerator];
	NSNumber *rowNum;

	while (nil != (rowNum = [theEnum nextObject]) )
	{
		id item = [oData objectAtIndex:[rowNum intValue]];
		[result addObject:item];
	}
	return result;
}

- (void) restoreSelection:(NSSet *)inSelectedItemNums toTable:(NSTableView *)inTableView
{
	NSEnumerator *theEnum = [inSelectedItemNums objectEnumerator];
	id item;
	int savedLastRow;

	[inTableView deselectAll:nil];

	while (nil != (item = [theEnum nextObject]) )
	{
		int row = [oData indexOfObjectIdenticalTo:item];	// look for an exact match, which is OK here.
		if (NSNotFound != row)
		{
			[inTableView selectRow:row byExtendingSelection:YES];
			savedLastRow = row;
		}
	}
	[inTableView scrollRowToVisible:savedLastRow];
}

// ----------------------------------------------------------------------------------------
// Sorting
// ----------------------------------------------------------------------------------------

- (void) sortData
{
	SortContext ctxt={ mSortingKey, mSortDescending };
	NSSet *oldSelection = [self saveSelectionFromTable:oTable];

	// sort the NSMutableArray
	[oData sortUsingFunction:ORDER_BY_CONTEXT context:&ctxt];

	[oTable reloadData];
	[self restoreSelection:oldSelection toTable:oTable];
}

- (void)sortByColumn:(NSTableColumn *)inTableColumn
{
    if (mSortingColumn == inTableColumn)
	{
        // User clicked same column, change sort order
        mSortDescending = !mSortDescending;
		// Possible optimization: Don't actually re-sort if you just change the sorting direction;
		// instead, just display either the nth item or the (count-1-n)th item depending on ascending/descending.)
    }
	else
	{
        // User clicked new column, change old/new column headers,
        // save new sorting column, and re-sort the array.
        mSortDescending = NO;
        if (nil != mSortingColumn)
		{
            [oTable setIndicatorImage: nil inTableColumn: mSortingColumn];
        }
		[self setSortingKey:[inTableColumn identifier]];
		[self setSortingColumn:inTableColumn];
		[oTable setHighlightedTableColumn:inTableColumn];
    }
    [oTable setIndicatorImage: (mSortDescending ?
							  [NSTableView descendingSortIndicator] :
							  [NSTableView ascendingSortIndicator])
								   inTableColumn: inTableColumn];
	// Actually sort the data
    [self sortData];
}

/*"	Sort by whatever column was clicked upon
"*/
- (void)tableView:(NSTableView*)inTableView
	didClickTableColumn:(NSTableColumn *)inTableColumn
{
	[[inTableView window] makeFirstResponder:inTableView];		// help make this tableView be first responder
	[self sortByColumn:inTableColumn];
}

// ----------------------------------------------------------------------------------------
// Alphabetic Type Ahead
// ----------------------------------------------------------------------------------------

- (void) typeAheadString:(NSString *)inString inTableView:(NSTableView *)inTableView
{
	// This general sample looks for a highlighted column, presuming that is that column we are sorted by, and uses that as the lookup key.
	NSTableColumn *col = [inTableView highlightedTableColumn];
	if (nil != col)
	{
		NSString *key = [col identifier];
		int i;
		for ( i = 0 ; i < [oData count] ; i++ )
		{
			NSDictionary *rowDict = [oData objectAtIndex:i];
			NSString *compareTo = [rowDict objectForKey:key];
			NSComparisonResult order = [inString caseInsensitiveCompare:compareTo];
			if (order != NSOrderedDescending)
			{
				break;
			}
		}
		// Make sure we're not overflowing the row count.
		if (i >= [oData count])
		{
			i = [oData count] - 1;
		}
		// Now select row i -- either the one we found, or the last row if not found.
		[inTableView selectRow:i byExtendingSelection:NO];
		[inTableView scrollRowToVisible:i];
	}
}




@end
